==> sd-shared-web-3.1.0-sources.jar: systems/dennis/shared/service/DeleteObject.java <==
package systems.dennis.shared.service;

import org.slf4j.Logger;
import systems.dennis.shared.annotations.DeleteStrategy;
import systems.dennis.shared.annotations.security.ISecurityUtils;
import systems.dennis.shared.entity.BaseEntity;
import systems.dennis.shared.exceptions.DeleteNotPossibleException;
import systems.dennis.shared.model.DeleteHistoryModel;
import systems.dennis.shared.repository.QueryCase;
import systems.dennis.shared.utils.bean_copier.BeanCopier;

import java.lang.reflect.Field;

import static systems.dennis.shared.entity.BaseEntity.*;

public interface DeleteObject<DB_TYPE extends BaseEntity> {

    default DB_TYPE delete(DB_TYPE object, AbstractService<DB_TYPE> service) {

        if (getSelfOnly(service.getClass())){
            service.getContext().getBean(ISecurityUtils.class).isMy(object);
        }

        int strategy = getDeleteStrategy(service.getClass());


        if (strategy == BaseEntity.DELETE_STRATEGY_DB) {
            service.getRepository().delete(object);
            saveDeleteHistoryIfRequired(object, service);
            return object;

        } else if (strategy == BaseEntity.DELETE_STRATEGY_PROPERTY) {
            Field field = BeanCopier.findField(getDeleteField(getClass()), object.getClass());;
            try {
                field.setAccessible(true);
                field.set(object, Boolean.TRUE);
                field.setAccessible(false);
                service.getRepository().save(object);
                saveDeleteHistoryIfRequired(object, service);
                return object;
            } catch (Exception e) {
                throw new DeleteNotPossibleException(e.getMessage(), field.getName());
            }

        }



        throw new DeleteNotPossibleException("global.wrong.delete.strategy", strategy);
    }

    /**
     * Save object after it was deleted from DB
     * if "global.save.deleted=true"
     *
     * @param deletedObject deleted object with ID
     */
    default void saveDeleteHistoryIfRequired(DB_TYPE deletedObject, AbstractService<DB_TYPE> service) {
        try {
            Boolean isRequired = getStoreHistoryRequired(service.getClass());
            if (isRequired) {
                var deleteHistoryModel = DeleteHistoryModel.from(deletedObject);
                service.getContext().getBean(DeleteHistoryService.class).save(deleteHistoryModel);
            }
        } catch (Exception e) {
            getLogger().error("Cannot save original version for object", e);
        }
    }

    Logger getLogger();

    default int getDeleteStrategy(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return DELETE_STRATEGY_DB;
        }
        return ann.value();
    }
    default boolean getSelfOnly(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return false;
        }
        return ann.selfOnly();
    }

    default String getDeleteField(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return DELETE_PROPERTY;
        }
        return ann.field();
    }

    default boolean getStoreHistoryRequired(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return false;
        }
        return ann.storeDeleteEventHistory();
    }

    /**
     * This method can be used as start point and will never produce null
     *
     * @return Query case
     */
    default QueryCase getNotDeletedQuery() {

        if (getDeleteStrategy(getClass()) == DELETE_STRATEGY_DB) {
            return QueryCase.ofNotNull("id");
        } else if (getDeleteStrategy(getClass()) == DELETE_STRATEGY_PROPERTY) {
            return QueryCase.notEqualsOf(getDeleteField(getClass()), true);
        }
        throw new DeleteNotPossibleException("global.wrong.delete.strategy", getDeleteStrategy(getClass()));
    }

}

==> sd-shared-web-3.1.3-sources.jar: systems/dennis/shared/service/DeleteObject.java <==
package systems.dennis.shared.service;

import org.slf4j.Logger;
import systems.dennis.shared.annotations.DeleteStrategy;
import systems.dennis.shared.annotations.security.ISecurityUtils;
import systems.dennis.shared.entity.AbstractEntity;
import systems.dennis.shared.exceptions.DeleteNotPossibleException;
import systems.dennis.shared.repository.QueryCase;
import systems.dennis.shared.utils.bean_copier.BeanCopier;

import java.io.Serializable;
import java.lang.reflect.Field;

import static systems.dennis.shared.annotations.DeleteStrategy.*;

public interface DeleteObject<DB_TYPE extends AbstractEntity<ID_TYPE>, ID_TYPE extends Serializable> {

    default DB_TYPE delete(DB_TYPE object, AbstractService<DB_TYPE, ID_TYPE> service) {

        if (getSelfOnly(service.getClass())){
            service.getContext().getBean(ISecurityUtils.class).isMy(object);
        }

        int strategy = getDeleteStrategy(service.getClass());


        if (strategy == DELETE_STRATEGY_DB) {
            service.getRepository().delete(object);
            saveDeleteHistoryIfRequired(object, service);
            return object;

        } else if (strategy ==  DELETE_STRATEGY_PROPERTY) {
            Field field = BeanCopier.findField(getDeleteField(getClass()), object.getClass());;
            try {
                field.setAccessible(true);
                field.set(object, Boolean.TRUE);
                field.setAccessible(false);
                service.getRepository().save(object);
                saveDeleteHistoryIfRequired(object, service);
                return object;
            } catch (Exception e) {
                throw new DeleteNotPossibleException(e.getMessage(), field.getName());
            }

        }



        throw new DeleteNotPossibleException("global.wrong.delete.strategy", strategy);
    }

    /**
     * Save object after it was deleted from DB
     * if "global.save.deleted=true"
     *
     * @param deletedObject deleted object with ID
     */
    default void saveDeleteHistoryIfRequired(DB_TYPE deletedObject, AbstractService<DB_TYPE, ID_TYPE> service) {
        try {
            //todo move
//            Boolean isRequired = getStoreHistoryRequired(service.getClass());
//            if (isRequired) {
//                var deleteHistoryModel = DeleteHistoryModel.from(deletedObject);
//                service.getContext().getBean(DeleteHistoryService.class).save(deleteHistoryModel);
//            }
        } catch (Exception e) {
            getLogger().error("Cannot save original version for object", e);
        }
    }

    Logger getLogger();

    default int getDeleteStrategy(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return DELETE_STRATEGY_DB;
        }
        return ann.value();
    }
    default boolean getSelfOnly(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return false;
        }
        return ann.selfOnly();
    }

    default String getDeleteField(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return DELETE_PROPERTY;
        }
        return ann.field();
    }

    default boolean getStoreHistoryRequired(Class<?> cl) {
        var ann = cl.getAnnotation(DeleteStrategy.class);
        if (ann == null) {
            return false;
        }
        return ann.storeDeleteEventHistory();
    }

    /**
     * This method can be used as start point and will never produce null
     *
     * @return Query case
     */
    default QueryCase getNotDeletedQuery() {

        if (getDeleteStrategy(getClass()) == DELETE_STRATEGY_DB) {
            return QueryCase.ofNotNull("id");
        } else if (getDeleteStrategy(getClass()) == DELETE_STRATEGY_PROPERTY) {
            return QueryCase.notEqualsOf(getDeleteField(getClass()), true);
        }
        throw new DeleteNotPossibleException("global.wrong.delete.strategy", getDeleteStrategy(getClass()));
    }

}
